@@ -61,5 +61,6 @@ def excess3_decode(text, errors="strict"):
     return r, len(b(text))
 
 
-add("excess3", excess3_encode, excess3_decode, pattern=r"^(?:excess\-?3|xs\-?3|stibitz)$")
+add("excess3", excess3_encode, excess3_decode, pattern=r"^(?:excess\-?3|xs\-?3|stibitz)$", entropy=1.,
+    printables_rate=.45)
 
